@@ -1,5 +1,4 @@
 #!/bin/sh
-set -o xtrace   # Write all commands first to stderr
 set -o errexit  # Exit the script with error if any of the commands fail
 
 # Supported/used environment variables:
@@ -18,6 +17,7 @@ set -o errexit  # Exit the script with error if any of the commands fail
 # Options for CMake:
 #       LIBBSON                 Build against bundled or external libbson
 #       EXTRA_CONFIGURE_FLAGS   Extra configure flags to use
+#       EXTRA_CMAKE_PREFIX_PATH Extra directories to search for libraries/packages
 #       ZLIB                    Build against bundled or external zlib, or none
 #       SNAPPY                  Build against Snappy, or none
 #       SSL                     Build against OpenSSL or native or none
@@ -71,6 +71,11 @@ fi
 # as an environment variable (e.g. to force 32bit)
 [ -z "$MARCH" ] && MARCH=$(uname -m | tr '[:upper:]' '[:lower:]')
 
+CMAKE_PREFIX_PATH="$INSTALL_DIR"
+if [ ! -z "$EXTRA_CMAKE_PREFIX_PATH" ]; then
+   CMAKE_PREFIX_PATH="$CMAKE_PREFIX_PATH;$EXTRA_CMAKE_PREFIX_PATH"
+fi
+
 # Default CMake flags for debug builds and release builds.
 # CMAKE_SKIP_RPATH avoids hardcoding absolute paths to dependency libraries.
 DEBUG_AND_RELEASE_FLAGS="\
@@ -82,7 +87,7 @@ DEBUG_AND_RELEASE_FLAGS="\
    -DENABLE_AUTOMATIC_INIT_AND_CLEANUP=OFF \
    -DENABLE_TRACING=$TRACING \
    -DENABLE_RDTSCP=$RDTSCP \
-   -DCMAKE_PREFIX_PATH=$INSTALL_DIR \
+   -DCMAKE_PREFIX_PATH=$CMAKE_PREFIX_PATH \
    -DCMAKE_INSTALL_PREFIX=$INSTALL_DIR \
    -DENABLE_SHM_COUNTERS=$ENABLE_SHM_COUNTERS \
 "
@@ -126,7 +131,11 @@ fi
 [ "$DEBUG" = "ON" ] && CONFIGURE_FLAGS=$DEBUG_FLAGS || CONFIGURE_FLAGS=$RELEASE_FLAGS
 
 CONFIGURE_FLAGS="$CONFIGURE_FLAGS -DENABLE_SASL=${SASL}"
-CONFIGURE_FLAGS="$CONFIGURE_FLAGS -DENABLE_SSL=${SSL}"
+if [ "${SSL}" = "OPENSSL_STATIC" ]; then
+   CONFIGURE_FLAGS="$CONFIGURE_FLAGS -DENABLE_SSL=OPENSSL -DOPENSSL_USE_STATIC_LIBS=ON"
+else
+   CONFIGURE_FLAGS="$CONFIGURE_FLAGS -DENABLE_SSL=${SSL}"
+fi
 [ "$COVERAGE" = "ON" ] && CONFIGURE_FLAGS="$CONFIGURE_FLAGS -DENABLE_COVERAGE=ON -DENABLE_EXAMPLES=OFF"
 
 if [ "$RELEASE" = "ON" ]; then
@@ -134,6 +143,8 @@ if [ "$RELEASE" = "ON" ]; then
    mkdir build-dir
    $TAR xf ../mongoc.tar.gz -C build-dir --strip-components=1
    cd build-dir
+else
+  CONFIGURE_FLAGS="${CONFIGURE_FLAGS} -DENABLE_DEBUG_ASSERTIONS=ON"
 fi
 
 if [ "$SRV" = "OFF" ]; then
@@ -151,7 +162,7 @@ case "$MARCH" in
    i386)
       CFLAGS="$CFLAGS -m32 -march=i386"
       CXXFLAGS="$CXXFLAGS -m32 -march=i386"
-      CONFIGURE_FLAGS="$CONFIGURE_FLAGS -DENABLE_SNAPPY=AUTO -DENABLE_ZLIB=BUNDLED"
+      CONFIGURE_FLAGS="$CONFIGURE_FLAGS -DENABLE_SNAPPY=AUTO -DENABLE_ZLIB=BUNDLED -DENABLE_ZSTD=OFF"
    ;;
    s390x)
       CFLAGS="$CFLAGS -march=z196 -mtune=zEC12"
@@ -184,6 +195,10 @@ case "$CC" in
    ;;
 esac
 
+if [ "darwin" = "$OS" -a "arm64" = "$MARCH" ]; then
+   CONFIGURE_FLAGS="$CONFIGURE_FLAGS -DCMAKE_OSX_ARCHITECTURES=arm64"
+fi
+
 CONFIGURE_FLAGS="$CONFIGURE_FLAGS $EXTRA_CONFIGURE_FLAGS"
 export MONGOC_TEST_FUTURE_TIMEOUT_MS=30000
 export MONGOC_TEST_SKIP_LIVE=on
@@ -206,7 +221,7 @@ if [ "$COMPILE_LIBMONGOCRYPT" = "ON" ]; then
    git clone https://github.com/mongodb/libmongocrypt
    mkdir libmongocrypt/cmake-build
    cd libmongocrypt/cmake-build
-   $CMAKE -DENABLE_SHARED_BSON=ON -DCMAKE_BUILD_TYPE="Debug" -DCMAKE_INSTALL_PREFIX="$INSTALL_DIR" -DCMAKE_PREFIX_PATH="$INSTALL_DIR" ../
+   $CMAKE -DENABLE_SHARED_BSON=ON -DCMAKE_BUILD_TYPE="Debug" -DCMAKE_INSTALL_PREFIX="$INSTALL_DIR" -DCMAKE_PREFIX_PATH="$CMAKE_PREFIX_PATH" ../
    make install
    cd ../../
    else
@@ -218,10 +233,18 @@ fi
 if [ "$ANALYZE" = "ON" ]; then
    # Clang static analyzer, available on Ubuntu 16.04 images.
    # https://clang-analyzer.llvm.org/scan-build.html
-   scan-build $CMAKE $CONFIGURE_FLAGS .
+   #
+   # On images other than Ubuntu 16.04, use scan-build-3.9 if
+   # scan-build is not found.
+   if command -v scan-build 2>/dev/null; then
+      SCAN_BUILD_COMMAND="scan-build"
+   else
+      SCAN_BUILD_COMMAND="scan-build-3.9"
+   fi
+   $SCAN_BUILD_COMMAND $CMAKE $CONFIGURE_FLAGS .
 
    # Put clang static analyzer results in scan/ and fail build if warnings found.
-   SCAN_BUILD="scan-build -o scan --status-bugs"
+   SCAN_BUILD="$SCAN_BUILD_COMMAND -o scan --status-bugs"
 else
    $CMAKE $CONFIGURE_FLAGS .
 fi
