@@ -6,7 +6,6 @@
 # Supported/used environment variables:
 #   IS_PATCH    If "true", this is an Evergreen patch build.
 
-set -o xtrace
 set -o errexit
 
 on_exit () {
@@ -43,7 +42,7 @@ git clone https://salsa.debian.org/installer-team/debootstrap.git debootstrap.gi
 export DEBOOTSTRAP_DIR=`pwd`/debootstrap.git
 sudo -E ./debootstrap.git/debootstrap unstable ./unstable-chroot/ http://cdn-aws.deb.debian.org/debian
 cp -a mongoc ./unstable-chroot/tmp/
-sudo chroot ./unstable-chroot /bin/bash -c "(set -o xtrace && \
+sudo chroot ./unstable-chroot /bin/bash -c "(\
   apt-get install -y build-essential git-buildpackage fakeroot debhelper cmake libssl-dev pkg-config python3-sphinx zlib1g-dev libicu-dev libsasl2-dev libsnappy-dev libzstd-dev libmongocrypt-dev && \
   cd /tmp/mongoc && \
   git clean -fdx && \
