@@ -2,7 +2,7 @@
 import os.path
 import sys
 
-# Ensure we can import "mongoc" and "taglist" extension modules.
+# Ensure we can import "mongoc" extension module.
 this_path = os.path.dirname(__file__)
 sys.path.append(this_path)
 sys.path.append(os.path.normpath(os.path.join(this_path, '../../../build/sphinx')))
@@ -11,7 +11,6 @@ from mongoc_common import *
 
 extensions = [
     'mongoc',
-    'taglist',
     'sphinx.ext.intersphinx',
 ]
 
