@@ -40,6 +40,7 @@ struct _mongoc_auto_encryption_opts_t {
    char *keyvault_db;
    char *keyvault_coll;
    bson_t *kms_providers;
+   bson_t *tls_opts;
    bson_t *schema_map;
    bool bypass_auto_encryption;
    bson_t *extra;
@@ -62,6 +63,7 @@ mongoc_auto_encryption_opts_destroy (mongoc_auto_encryption_opts_t *opts)
    bson_destroy (opts->schema_map);
    bson_free (opts->keyvault_db);
    bson_free (opts->keyvault_coll);
+   bson_destroy (opts->tls_opts);
    bson_free (opts);
 }
 
@@ -117,6 +119,27 @@ mongoc_auto_encryption_opts_set_kms_providers (
    }
 }
 
+/* _bson_copy_or_null returns a copy of @bson or NULL if @bson is NULL */
+static bson_t *
+_bson_copy_or_null (const bson_t *bson)
+{
+   if (bson) {
+      return bson_copy (bson);
+   }
+   return NULL;
+}
+
+void
+mongoc_auto_encryption_opts_set_tls_opts (mongoc_auto_encryption_opts_t *opts,
+                                          const bson_t *tls_opts)
+{
+   if (!opts) {
+      return;
+   }
+   bson_destroy (opts->tls_opts);
+   opts->tls_opts = _bson_copy_or_null (tls_opts);
+}
+
 void
 mongoc_auto_encryption_opts_set_schema_map (mongoc_auto_encryption_opts_t *opts,
                                             const bson_t *schema_map)
@@ -165,6 +188,7 @@ struct _mongoc_client_encryption_opts_t {
    char *keyvault_db;
    char *keyvault_coll;
    bson_t *kms_providers;
+   bson_t *tls_opts;
 };
 
 mongoc_client_encryption_opts_t *
@@ -182,6 +206,7 @@ mongoc_client_encryption_opts_destroy (mongoc_client_encryption_opts_t *opts)
    bson_free (opts->keyvault_db);
    bson_free (opts->keyvault_coll);
    bson_destroy (opts->kms_providers);
+   bson_destroy (opts->tls_opts);
    bson_free (opts);
 }
 
@@ -224,6 +249,17 @@ mongoc_client_encryption_opts_set_kms_providers (
    }
 }
 
+void
+mongoc_client_encryption_opts_set_tls_opts (
+   mongoc_client_encryption_opts_t *opts, const bson_t *tls_opts)
+{
+   if (!opts) {
+      return;
+   }
+   bson_destroy (opts->tls_opts);
+   opts->tls_opts = _bson_copy_or_null (tls_opts);
+}
+
 /*--------------------------------------------------------------------------
  * Data key options.
  *--------------------------------------------------------------------------
@@ -945,9 +981,10 @@ _do_spawn (const char *path, char **args, bson_error_t *error)
       exit (EXIT_SUCCESS);
    }
 
-   /* If we later decide to change the working directory for the pid file path, possibly change the
-    * process's working directory with chdir like: `chdir (default_pid_path)`.
-    * Currently pid file ends up in application's working directory. */
+   /* If we later decide to change the working directory for the pid file path,
+    * possibly change the process's working directory with chdir like: `chdir
+    * (default_pid_path)`. Currently pid file ends up in application's working
+    * directory. */
 
    /* Set the user file creation mask to zero. */
    umask (0);
@@ -1025,6 +1062,7 @@ _spawn_mongocryptd (const char *mongocryptd_spawn_path,
    bool passed_idle_shutdown_timeout_secs = false;
    int num_args = 2; /* for leading "mongocrypt" and trailing NULL */
    int i;
+   bool ret;
 
    /* iterate once to get length and validate all are strings */
    if (mongocryptd_spawn_args) {
@@ -1073,7 +1111,9 @@ _spawn_mongocryptd (const char *mongocryptd_spawn_path,
    BSON_ASSERT (i == num_args - 1);
    args[i++] = NULL;
 
-   return _do_spawn (mongocryptd_spawn_path, args, error);
+   ret = _do_spawn (mongocryptd_spawn_path, args, error);
+   bson_free (args);
+   return ret;
 }
 
 static bool
@@ -1152,7 +1192,7 @@ _parse_extra (const bson_t *extra,
       }
 
       if (!mongoc_uri_set_option_as_int32 (
-             *uri, MONGOC_URI_SERVERSELECTIONTIMEOUTMS, 5000)) {
+             *uri, MONGOC_URI_SERVERSELECTIONTIMEOUTMS, 10000)) {
          _uri_construction_error (error);
          GOTO (fail);
       }
@@ -1227,22 +1267,22 @@ _mongoc_cse_client_enable_auto_encryption (mongoc_client_t *client,
       GOTO (fail);
    }
 
-   if (client->topology->cse_enabled) {
+   if (client->topology->cse_state != MONGOC_CSE_DISABLED) {
       bson_set_error (error,
                       MONGOC_ERROR_CLIENT,
                       MONGOC_ERROR_CLIENT_INVALID_ENCRYPTION_STATE,
                       "Automatic encryption already set");
       GOTO (fail);
    } else {
-      client->topology->cse_enabled = true;
+      client->topology->cse_state = MONGOC_CSE_ENABLED;
    }
 
    if (!_parse_extra (opts->extra, client->topology, &mongocryptd_uri, error)) {
       GOTO (fail);
    }
 
-   client->topology->crypt =
-      _mongoc_crypt_new (opts->kms_providers, opts->schema_map, error);
+   client->topology->crypt = _mongoc_crypt_new (
+      opts->kms_providers, opts->schema_map, opts->tls_opts, error);
    if (!client->topology->crypt) {
       GOTO (fail);
    }
@@ -1289,10 +1329,10 @@ _mongoc_cse_client_enable_auto_encryption (mongoc_client_t *client,
          client->topology->mongocryptd_client->topology);
 
       /* Also, since single threaded server selection can foreseeably take
-       * connectTimeoutMS (which by default is longer than 5 seconds), reduce
+       * connectTimeoutMS (which by default is longer than 10 seconds), reduce
        * this as well. */
       if (!mongoc_uri_set_option_as_int32 (
-             mongocryptd_uri, MONGOC_URI_CONNECTTIMEOUTMS, 5000)) {
+             mongocryptd_uri, MONGOC_URI_CONNECTTIMEOUTMS, 10000)) {
          _uri_construction_error (error);
          GOTO (fail);
       }
@@ -1316,11 +1356,11 @@ _mongoc_cse_client_pool_enable_auto_encryption (
    mongoc_auto_encryption_opts_t *opts,
    bson_error_t *error)
 {
-   bool ret = false;
+   bool setup_okay = false;
    mongoc_uri_t *mongocryptd_uri = NULL;
+   mongoc_topology_cse_state_t prev_cse_state = MONGOC_CSE_STARTING;
 
    BSON_ASSERT (topology);
-   bson_mutex_lock (&topology->mutex);
    if (!opts) {
       bson_set_error (error,
                       MONGOC_ERROR_CLIENT,
@@ -1355,22 +1395,38 @@ _mongoc_cse_client_pool_enable_auto_encryption (
       GOTO (fail);
    }
 
-   if (topology->cse_enabled) {
+   prev_cse_state =
+      bson_atomic_int_compare_exchange_strong ((int *) &topology->cse_state,
+                                               MONGOC_CSE_DISABLED,
+                                               MONGOC_CSE_STARTING,
+                                               bson_memory_order_acquire);
+   while (prev_cse_state == MONGOC_CSE_STARTING) {
+      /* Another thread is starting client-side encryption. It may take some
+       * time to start, but don't continue until it is finished. */
+      bson_thrd_yield ();
+      prev_cse_state =
+         bson_atomic_int_compare_exchange_strong ((int *) &topology->cse_state,
+                                                  MONGOC_CSE_DISABLED,
+                                                  MONGOC_CSE_STARTING,
+                                                  bson_memory_order_acquire);
+   }
+
+   if (prev_cse_state == MONGOC_CSE_ENABLED) {
       bson_set_error (error,
                       MONGOC_ERROR_CLIENT,
                       MONGOC_ERROR_CLIENT_INVALID_ENCRYPTION_STATE,
                       "Automatic encryption already set");
       GOTO (fail);
-   } else {
-      topology->cse_enabled = true;
    }
 
+   /* We just set the CSE state from DISABLED to STARTING. Start it up now. */
+
    if (!_parse_extra (opts->extra, topology, &mongocryptd_uri, error)) {
       GOTO (fail);
    }
 
-   topology->crypt =
-      _mongoc_crypt_new (opts->kms_providers, opts->schema_map, error);
+   topology->crypt = _mongoc_crypt_new (
+      opts->kms_providers, opts->schema_map, opts->tls_opts, error);
    if (!topology->crypt) {
       GOTO (fail);
    }
@@ -1404,11 +1460,18 @@ _mongoc_cse_client_pool_enable_auto_encryption (
       topology->keyvault_client_pool = opts->keyvault_client_pool;
    }
 
-   ret = true;
+   setup_okay = true;
+   BSON_ASSERT (prev_cse_state == MONGOC_CSE_DISABLED);
 fail:
+   if (prev_cse_state == MONGOC_CSE_DISABLED) {
+      /* We need to set the new CSE state. */
+      mongoc_topology_cse_state_t new_state =
+         setup_okay ? MONGOC_CSE_ENABLED : MONGOC_CSE_DISABLED;
+      bson_atomic_int_exchange (
+         (int *) &topology->cse_state, new_state, bson_memory_order_release);
+   }
    mongoc_uri_destroy (mongocryptd_uri);
-   bson_mutex_unlock (&topology->mutex);
-   RETURN (ret);
+   RETURN (setup_okay);
 }
 
 struct _mongoc_client_encryption_t {
@@ -1451,8 +1514,8 @@ mongoc_client_encryption_new (mongoc_client_encryption_opts_t *opts,
    mongoc_collection_set_write_concern (client_encryption->keyvault_coll, wc);
 
    client_encryption->kms_providers = bson_copy (opts->kms_providers);
-   client_encryption->crypt =
-      _mongoc_crypt_new (opts->kms_providers, NULL /* schema_map */, error);
+   client_encryption->crypt = _mongoc_crypt_new (
+      opts->kms_providers, NULL /* schema_map */, opts->tls_opts, error);
    if (!client_encryption->crypt) {
       goto fail;
    }
@@ -1655,12 +1718,15 @@ fail:
 bool
 _mongoc_cse_is_enabled (mongoc_client_t *client)
 {
-   bool ret = false;
-
-   bson_mutex_lock (&client->topology->mutex);
-   ret = client->topology->cse_enabled;
-   bson_mutex_unlock (&client->topology->mutex);
-   return ret;
+   while (1) {
+      mongoc_topology_cse_state_t state = bson_atomic_int_fetch (
+         (int *) &client->topology->cse_state, bson_memory_order_relaxed);
+      if (state != MONGOC_CSE_STARTING) {
+         return state == MONGOC_CSE_ENABLED;
+      }
+      /* CSE is starting up. Wait until that succeeds or fails. */
+      bson_thrd_yield ();
+   }
 }
 
 #endif /* MONGOC_ENABLE_CLIENT_SIDE_ENCRYPTION */
