@@ -1043,11 +1043,11 @@ _mongoc_rpc_prep_command (mongoc_rpc_t *rpc,
    rpc->query.query = bson_get_data (cmd->command);
 
    /* Find, getMore And killCursors Commands Spec: "When sending a find command
-    * rather than a legacy OP_QUERY find, only the slaveOk flag is honored."
-    * For other cursor-typed commands like aggregate, only slaveOk can be set.
-    * Clear bits except slaveOk; leave slaveOk set only if it is already.
+    * rather than a legacy OP_QUERY find, only the secondaryOk flag is honored."
+    * For other cursor-typed commands like aggregate, only secondaryOk can be set.
+    * Clear bits except secondaryOk; leave secondaryOk set only if it is already.
     */
-   rpc->query.flags = cmd->query_flags & MONGOC_QUERY_SLAVE_OK;
+   rpc->query.flags = cmd->query_flags & MONGOC_QUERY_SECONDARY_OK;
 }
 
 
