@@ -30,7 +30,7 @@ MONGOC_EXPORT (mongoc_stream_t *)
 mongoc_stream_tls_openssl_new (mongoc_stream_t *base_stream,
                                const char *host,
                                mongoc_ssl_opt_t *opt,
-                               int client);
+                               int client) BSON_GNUC_WARN_UNUSED_RESULT;
 
 BSON_END_DECLS
 
