@@ -113,45 +113,6 @@ static BSON_ONCE_FUN (_test_suite_ensure_mutex_once)
 }
 
 
-static void
-_handle_signal (int signum)
-{
-   const char *s = "\nProcess was interrupted by the delivery of a signal.\n";
-   const char *sigstr;
-   size_t n;
-   switch (signum) {
-   case SIGABRT:
-      sigstr = "SIGABRT - Abnormal termination";
-      break;
-   case SIGINT:
-      sigstr = "SIGINT - Interrupted";
-      break;
-   case SIGTERM:
-      sigstr = "SIGTERM - Termination requested";
-      break;
-   case SIGSEGV:
-      sigstr = "SIGSEGV - Access violation";
-      break;
-   default:
-      sigstr = "(Unknown signal delivered)";
-   }
-#ifdef BSON_OS_UNIX
-   /* On POSIX these APIs are signal-safe */
-   n = write (STDERR_FILENO, s, strlen (s));
-   n = write (STDERR_FILENO, "  ", 2);
-   n = write (STDERR_FILENO, sigstr, strlen (sigstr));
-   n = write (STDERR_FILENO, "\n", 1);
-   fsync (STDERR_FILENO);
-#else
-   /* On Windows these APIs are signal-safe */
-   fprintf (stderr, "\n%s\n  %s\n", s, sigstr);
-   fflush (stderr);
-#endif
-   _Exit (signum);
-   (void) n;
-}
-
-
 void
 TestSuite_Init (TestSuite *suite, const char *name, int argc, char **argv)
 {
@@ -168,11 +129,6 @@ TestSuite_Init (TestSuite *suite, const char *name, int argc, char **argv)
    suite->ctest_run = NULL;
    _mongoc_array_init (&suite->match_patterns, sizeof (char *));
 
-   suite->prev_sigabrt = signal (SIGABRT, _handle_signal);
-   suite->prev_sigint = signal (SIGINT, _handle_signal);
-   suite->prev_sigterm = signal (SIGTERM, _handle_signal);
-   suite->prev_sigsegv = signal (SIGSEGV, _handle_signal);
-
    for (i = 1; i < argc; i++) {
       if (0 == strcmp ("-d", argv[i])) {
          suite->flags |= TEST_DEBUGOUTPUT;
@@ -181,11 +137,11 @@ TestSuite_Init (TestSuite *suite, const char *name, int argc, char **argv)
          suite->flags |= TEST_NOFORK;
       } else if ((0 == strcmp ("-t", argv[i])) ||
                  (0 == strcmp ("--trace", argv[i]))) {
-#ifdef MONGOC_TRACE
+         if (!MONGOC_TRACE_ENABLED) {
+            test_error (
+               "-t requires mongoc compiled with -DENABLE_TRACING=ON.");
+         }
          suite->flags |= TEST_TRACE;
-#else
-         test_error ("-t requires mongoc compiled with -DENABLE_TRACING=ON.");
-#endif
       } else if (0 == strcmp ("-F", argv[i])) {
          if (argc - 1 == i) {
             test_error ("-F requires a filename argument.");
@@ -353,6 +309,10 @@ _V_TestSuite_AddFull (TestSuite *suite,
    Test *test;
    Test *iter;
 
+   if (suite->ctest_run && (0 != strcmp (suite->ctest_run, name))) {
+      return NULL;
+   }
+
    test = (Test *) calloc (1, sizeof *test);
    test->name = bson_strdup (name);
    test->func = func;
@@ -393,7 +353,9 @@ _TestSuite_AddMockServerTest (TestSuite *suite,
    test = _V_TestSuite_AddFull (suite, name, (TestFuncWC) func, NULL, NULL, ap);
    va_end (ap);
 
-   _TestSuite_AddCheck (test, TestSuite_CheckMockServerAllowed, name);
+   if (test) {
+      _TestSuite_AddCheck (test, TestSuite_CheckMockServerAllowed, name);
+   }
 }
 
 
@@ -446,7 +408,8 @@ _print_getlasterror_win (const char *msg)
                   NULL,
                   GetLastError (),
                   MAKELANGID (LANG_NEUTRAL, SUBLANG_DEFAULT),
-                  &err_msg,
+                  /* FormatMessage is weird about this param. */
+                  (LPTSTR) &err_msg,
                   0,
                   NULL);
 
@@ -644,14 +607,12 @@ TestSuite_RunTest (TestSuite *suite, /* IN */
 
 
    if (suite->flags & TEST_NOFORK) {
-#ifdef MONGOC_TRACE
       if (suite->flags & TEST_TRACE) {
          mongoc_log_set_handler (mongoc_log_default_handler, NULL);
          mongoc_log_trace_enable ();
       } else {
          mongoc_log_trace_disable ();
       }
-#endif
 
       srand (test->seed);
       test_conveniences_init ();
@@ -1095,11 +1056,6 @@ TestSuite_Destroy (TestSuite *suite)
    }
 
    _mongoc_array_destroy (&suite->match_patterns);
-
-   signal (SIGABRT, suite->prev_sigabrt);
-   signal (SIGINT, suite->prev_sigint);
-   signal (SIGTERM, suite->prev_sigterm);
-   signal (SIGSEGV, suite->prev_sigsegv);
 }
 
 
