@@ -2107,20 +2107,18 @@ index_exists (mongoc_client_t *client, const bson_t *operation)
 static uint32_t
 _get_total_pool_cleared_event (json_test_ctx_t *ctx)
 {
-   mongoc_topology_description_t *td;
    uint32_t i;
    uint32_t total = 0;
+   mc_shared_tpld td = mc_tpld_take_ref (ctx->client->topology);
 
    /* Go get total generation counts. */
-   bson_mutex_lock (&ctx->client->topology->mutex);
-   td = &ctx->client->topology->description;
-   for (i = 0; i < td->servers->items_len; i++) {
-      mongoc_server_description_t *sd;
+   for (i = 0; i < mc_tpld_servers_const (td.ptr)->items_len; i++) {
+      const mongoc_server_description_t *sd;
 
-      sd = mongoc_set_get_item (td->servers, i);
-      total += mongoc_generation_map_get (sd->generation_map, &kZeroServiceId);
+      sd = mongoc_set_get_item_const (mc_tpld_servers_const (td.ptr), i);
+      total += mc_tpl_sd_get_generation (sd, &kZeroServiceId);
    }
-   bson_mutex_unlock (&ctx->client->topology->mutex);
+   mc_tpld_drop_ref (&td);
    return total;
 }
 
@@ -2156,14 +2154,8 @@ wait_for_event (json_test_ctx_t *ctx, const bson_t *operation)
          }
          bson_mutex_unlock (&ctx->mutex);
       } else if (0 == strcmp (event_name, "PoolClearedEvent")) {
-         /* Do *NOT* lock ctx->mutex while calling
-          * _get_total_pool_cleared_event, which locks the topology mutex.
-          * This could create a deadlock situation.
-          * A monitor may be updating the topology description, calling an APM
-          * callback: topology->mutex => ctx->mutex
-          */
-         total = _get_total_pool_cleared_event (ctx);
          bson_mutex_lock (&ctx->mutex);
+         total = _get_total_pool_cleared_event (ctx);
          measured = ctx->measured_PoolClearedEvent;
          diff = total - measured;
          if (diff >= count) {
