@@ -502,6 +502,7 @@ test_server_selection_logic_cb (bson_t *test)
       op,
       &topology,
       read_prefs,
+      NULL,
       MONGOC_TOPOLOGY_LOCAL_THRESHOLD_MS);
 
    /* check each server in expected_servers is in selected_servers */
@@ -1178,7 +1179,7 @@ execute_test (const json_test_config_t *config,
 
    /* Select a primary for testing */
    server_id = mongoc_topology_select_server_id (
-      client->topology, MONGOC_SS_WRITE, NULL, &error);
+      client->topology, MONGOC_SS_WRITE, NULL, NULL, &error);
    ASSERT_OR_PRINT (server_id, error);
 
    json_test_ctx_init (&ctx, test, client, db, collection, config);
@@ -1790,7 +1791,7 @@ run_json_general_test (const json_test_config_t *config)
 
       /* clean up in case a previous test aborted */
       server_id = mongoc_topology_select_server_id (
-         client->topology, MONGOC_SS_WRITE, NULL, &error);
+         client->topology, MONGOC_SS_WRITE, NULL, NULL, &error);
       ASSERT_OR_PRINT (server_id, error);
       deactivate_fail_points (client, server_id);
       r = mongoc_client_command_with_opts (client,
@@ -1915,7 +1916,8 @@ _skip_if_unsupported (const char *test_name, bson_t *original)
  */
 void
 _install_json_test_suite_with_check (TestSuite *suite,
-                                     const char *dir_path,
+                                     const char *base,
+                                     const char *subdir,
                                      test_hook callback,
                                      ...)
 {
@@ -1926,9 +1928,21 @@ _install_json_test_suite_with_check (TestSuite *suite,
    char *skip_json;
    char *ext;
    va_list ap;
+   char joined[PATH_MAX];
+   char resolved[PATH_MAX];
+
+   bson_snprintf (joined, PATH_MAX, "%s/%s", base, subdir);
+   ASSERT (realpath (joined, resolved));
+
+   if (suite->ctest_run) {
+      const char *found = strstr (suite->ctest_run, subdir);
+      if (found != suite->ctest_run && found != suite->ctest_run + 1) {
+         return;
+      }
+   }
 
    num_tests =
-      collect_tests_from_dir (&test_paths[0], dir_path, 0, MAX_NUM_TESTS);
+      collect_tests_from_dir (&test_paths[0], resolved, 0, MAX_NUM_TESTS);
 
    for (i = 0; i < num_tests; i++) {
       test = get_bson_from_json_file (test_paths[i]);
@@ -1971,9 +1985,10 @@ _install_json_test_suite_with_check (TestSuite *suite,
  */
 void
 install_json_test_suite (TestSuite *suite,
-                         const char *dir_path,
+                         const char *base,
+                         const char *subdir,
                          test_hook callback)
 {
    install_json_test_suite_with_check (
-      suite, dir_path, callback, TestSuite_CheckLive);
+      suite, base, subdir, callback, TestSuite_CheckLive);
 }
